from abc import ABC

from rag4p.indexing.content_reader import ContentReader
from rag4p.indexing.input_document import InputDocument
from rag4p.rag.store.content_store import ContentStore
from rag4p.indexing.splitter import Splitter


class IndexingService(ABC):
    def __init__(self, content_store: ContentStore):
        self.content_store = content_store

    def index_documents(self, content_reader: ContentReader, splitter: Splitter):
        for batch in content_reader.read():
            print(f"Indexing batch of size {len(batch)}")
            for document in batch:
                self.index_document(document, splitter)

    def index_document(self, document: InputDocument, splitter: Splitter):
        chunks = splitter.split(document)
        self.content_store.store(chunks)



EMBEDDING_MODEL_NOMIC = "nomic-embed-text"
EMBEDDING_MODEL_MINILM = "all-minilm"

MODEL_PHI3 = "phi3"
MODEL_LLAMA3 = "llama3"

DEFAULT_MODEL = MODEL_PHI3
DEFAULT_EMBEDDING_MODEL = EMBEDDING_MODEL_NOMIC




from typing import List

import tiktoken

from rag4p.rag.model.chunk import Chunk
from rag4p.indexing.input_document import InputDocument
from rag4p.indexing.splitter import Splitter
from rag4p.integrations.openai import DEFAULT_EMBEDDING_MODEL


class MaxTokenSplitter(Splitter):
    """
    Splits an InputDocument into Chunks of a maximum number of tokens. The tokens are obtained by
    encoding the text of the document using the default model from openai encoding. The chunks of tokens are
    decoded back into text.
    """
    def __init__(self, max_tokens: int = 200, model: str = DEFAULT_EMBEDDING_MODEL):
        self.max_tokens = max_tokens
        self.encoding = tiktoken.encoding_for_model(model)

    def split(self, input_document: InputDocument) -> List[Chunk]:
        tokens = self.encoding.encode(input_document.text)

        chunks = []
        num_chunks = len(tokens) // self.max_tokens + (len(tokens) % self.max_tokens != 0)

        while tokens:
            chunk_size = min(len(tokens), self.max_tokens)
            chunk_tokens = tokens[:chunk_size]
            tokens = tokens[chunk_size:]
            chunk_text = self.encoding.decode(chunk_tokens)
            chunk = Chunk(input_document.document_id, len(chunks), num_chunks, chunk_text, input_document.properties)
            chunks.append(chunk)

        return chunks
